@@ -16,9 +16,7 @@ from __future__ import annotations
 
 from dataclasses import dataclass
 from textwrap import dedent
-from typing import TYPE_CHECKING, Literal, Union, cast, overload
-
-from typing_extensions import TypeAlias
+from typing import TYPE_CHECKING, Literal, TypeAlias, cast, overload
 
 from streamlit import config
 from streamlit.elements.lib.file_uploader_utils import (
@@ -60,12 +58,15 @@ if TYPE_CHECKING:
 
     from streamlit.delta_generator import DeltaGenerator
 
-SomeUploadedFiles: TypeAlias = Union[
-    UploadedFile,
-    DeletedFile,
-    list[Union[UploadedFile, DeletedFile]],
-    None,
-]
+SomeUploadedFiles: TypeAlias = (
+    UploadedFile | DeletedFile | list[UploadedFile | DeletedFile] | None
+)
+
+# Type alias for accept_multiple_files parameter.
+# If True, multiple files can be uploaded.
+# If False, only a single file can be uploaded.
+# If set to the literal "directory", users can upload an entire directory (folder) of files.
+AcceptMultipleFiles: TypeAlias = bool | Literal["directory"]
 
 
 def _get_upload_files(
@@ -104,7 +105,7 @@ def _get_upload_files(
 
 @dataclass
 class FileUploaderSerde:
-    accept_multiple_files: bool
+    accept_multiple_files: AcceptMultipleFiles
     allowed_types: Sequence[str] | None = None
 
     def deserialize(self, ui_value: FileUploaderStateProto | None) -> SomeUploadedFiles:
@@ -117,12 +118,16 @@ class FileUploaderSerde:
             if self.allowed_types:
                 enforce_filename_restriction(file.name, self.allowed_types)
 
+        # Directory uploads always return a list, similar to multiple files
+        is_multiple_or_directory = (
+            self.accept_multiple_files is True
+            or self.accept_multiple_files == "directory"
+        )
+
         if len(upload_files) == 0:
-            return_value: SomeUploadedFiles = [] if self.accept_multiple_files else None
+            return_value: SomeUploadedFiles = [] if is_multiple_or_directory else None
         else:
-            return_value = (
-                upload_files if self.accept_multiple_files else upload_files[0]
-            )
+            return_value = upload_files if is_multiple_or_directory else upload_files[0]
         return return_value
 
     def serialize(self, files: SomeUploadedFiles) -> FileUploaderStateProto:
@@ -149,21 +154,22 @@ class FileUploaderMixin:
     # Multiple overloads are defined on `file_uploader()` below to represent
     # the different return types of `file_uploader()`.
     # These return types differ according to the value of the `accept_multiple_files` argument.
-    # There are 2 associated variables, each with 2 options.
-    # 1. The `accept_multiple_files` argument is set as `True`,
-    #    or it is set as `False` or omitted, in which case the default value `False`.
-    # 2. The `type` argument may or may not be provided as a keyword-only argument.
     # There must be 2x2=4 overloads to cover all the possible arguments,
     # as these overloads must be mutually exclusive for mypy.
+    # There are 3 associated variables, each with 2+ options.
+    # 1. The `accept_multiple_files` argument is set as `True` or `"directory"`,
+    #    or it is set as `False` or omitted, in which case the default value `False`.
+    # 2. The `type` argument may or may not be provided as a keyword-only argument.
+    # 3. Directory uploads always return a list of UploadedFile objects.
 
     # 1. type is given as not a keyword-only argument
-    # 2. accept_multiple_files = True
+    # 2. accept_multiple_files = True or "directory"
     @overload
     def file_uploader(
         self,
         label: str,
         type: str | Sequence[str] | None,
-        accept_multiple_files: Literal[True],
+        accept_multiple_files: Literal[True, "directory"],
         key: Key | None = None,
         help: str | None = None,
         on_change: WidgetCallback | None = None,
@@ -200,13 +206,13 @@ class FileUploaderMixin:
     # for the related discussions and examples.
 
     # 1. type is skipped or a keyword argument
-    # 2. accept_multiple_files = True
+    # 2. accept_multiple_files = True or "directory"
     @overload
     def file_uploader(
         self,
         label: str,
         *,
-        accept_multiple_files: Literal[True],
+        accept_multiple_files: Literal[True, "directory"],
         type: str | Sequence[str] | None = None,
         key: Key | None = None,
         help: str | None = None,
@@ -242,7 +248,7 @@ class FileUploaderMixin:
         self,
         label: str,
         type: str | Sequence[str] | None = None,
-        accept_multiple_files: bool = False,
+        accept_multiple_files: AcceptMultipleFiles = False,
         key: Key | None = None,
         help: str | None = None,
         on_change: WidgetCallback | None = None,
@@ -285,7 +291,7 @@ class FileUploaderMixin:
             .. |st.markdown| replace:: ``st.markdown``
             .. _st.markdown: https://docs.streamlit.io/develop/api-reference/text/st.markdown
 
-        type : str or list of str or None
+        type : str, list of str, or None
             The allowed file extension(s) for uploaded files. This can be one
             of the following types:
 
@@ -302,11 +308,19 @@ class FileUploaderMixin:
                 or type extensions. The correct handling of uploaded files is
                 part of the app developer's responsibility.
 
-        accept_multiple_files : bool
-            Whether to accept more than one file in a submission. If this is
-            ``False`` (default), the user can only submit one file at a time.
-            If this is ``True``, the user can upload multiple files at the same
-            time, in which case the return value will be a list of files.
+        accept_multiple_files : bool or "directory"
+            Whether to accept more than one file in a submission. This can be one
+            of the following values:
+
+            - ``False`` (default): The user can only submit one file at a time.
+            - ``True``: The user can upload multiple files at the same time.
+            - ``"directory"``: The user can select a directory to upload all
+              files in the directory and its subdirectories. If ``type`` is
+              set, only files matching those type(s) will be uploaded.
+
+            When this is ``True`` or ``"directory"``, the return value will be
+            a list and a user can additively select files if they click the
+            browse button on the widget multiple times.
 
         key : str or int
             An optional string or integer to use as the unique key for the widget.
@@ -356,19 +370,19 @@ class FileUploaderMixin:
         Returns
         -------
         None, UploadedFile, or list of UploadedFile
-            - If accept_multiple_files is False, returns either None or
-              an UploadedFile object.
-            - If accept_multiple_files is True, returns a list with the
-              uploaded files as UploadedFile objects. If no files were
-              uploaded, returns an empty list.
+            - If accept_multiple_files is ``False``, returns either ``None`` or
+              an ``UploadedFile`` object.
+            - If accept_multiple_files is ``True`` or ``"directory"``, returns
+              a list with the uploaded files as ``UploadedFile`` objects. If no
+              files were uploaded, returns an empty list.
 
-            The UploadedFile class is a subclass of BytesIO, and therefore is
-            "file-like". This means you can pass an instance of it anywhere a
-            file is expected.
+            The ``UploadedFile`` class is a subclass of ``BytesIO``, and
+            therefore is "file-like". This means you can pass an instance of it
+            anywhere a file is expected.
 
         Examples
         --------
-        Insert a file uploader that accepts a single file at a time:
+        **Example 1: Accept a single file at a time**
 
         >>> import streamlit as st
         >>> import pandas as pd
@@ -392,20 +406,34 @@ class FileUploaderMixin:
         ...     dataframe = pd.read_csv(uploaded_file)
         ...     st.write(dataframe)
 
-        Insert a file uploader that accepts multiple files at a time:
+        **Example 2: Accept multiple files at a time**
+
+        >>> import pandas as pd
+        >>> import streamlit as st
+        >>>
+        >>> uploaded_files = st.file_uploader(
+        ...     "Upload data", accept_multiple_files=True, type="csv"
+        ... )
+        >>> for uploaded_file in uploaded_files:
+        ...     df = pd.read_csv(uploaded_file)
+        ...     st.write(df)
+
+        .. output::
+           https://doc-file-uploader.streamlit.app/
+           height: 375px
+
+        **Example 3: Accept an entire directory**
 
         >>> import streamlit as st
         >>>
         >>> uploaded_files = st.file_uploader(
-        ...     "Choose a CSV file", accept_multiple_files=True
+        ...     "Upload images", accept_multiple_files="directory", type=["jpg", "png"]
         ... )
         >>> for uploaded_file in uploaded_files:
-        ...     bytes_data = uploaded_file.read()
-        ...     st.write("filename:", uploaded_file.name)
-        ...     st.write(bytes_data)
+        ...     st.image(uploaded_file)
 
         .. output::
-           https://doc-file-uploader.streamlit.app/
+           https://doc-file-uploader-directory.streamlit.app/
            height: 375px
 
         """
@@ -429,7 +457,7 @@ class FileUploaderMixin:
         self,
         label: str,
         type: str | Sequence[str] | None = None,
-        accept_multiple_files: bool = False,
+        accept_multiple_files: AcceptMultipleFiles = False,
         key: Key | None = None,
         help: str | None = None,
         on_change: WidgetCallback | None = None,
@@ -455,7 +483,7 @@ class FileUploaderMixin:
         element_id = compute_and_register_element_id(
             "file_uploader",
             user_key=key,
-            form_id=current_form_id(self.dg),
+            key_as_main_identity=False,
             dg=self.dg,
             label=label,
             type=type,
@@ -475,7 +503,12 @@ class FileUploaderMixin:
         file_uploader_proto.max_upload_size_mb = config.get_option(
             "server.maxUploadSize"
         )
-        file_uploader_proto.multiple_files = accept_multiple_files
+        # Handle directory uploads - they should enable multiple files and set the directory flag
+        is_directory_upload = accept_multiple_files == "directory"
+        file_uploader_proto.multiple_files = (
+            accept_multiple_files is True or is_directory_upload
+        )
+        file_uploader_proto.accept_directory = is_directory_upload
         file_uploader_proto.form_id = current_form_id(self.dg)
         file_uploader_proto.disabled = disabled
         file_uploader_proto.label_visibility.value = get_label_visibility_proto_value(
