@@ -19,17 +19,18 @@ from __future__ import annotations
 
 import math
 import threading
+from collections.abc import Callable
 from typing import (
     TYPE_CHECKING,
     Any,
-    Callable,
     Final,
+    TypeAlias,
     TypeVar,
     overload,
 )
 
 from cachetools import TTLCache
-from typing_extensions import ParamSpec, TypeAlias
+from typing_extensions import ParamSpec
 
 import streamlit as st
 from streamlit.logger import get_logger
@@ -46,7 +47,6 @@ from streamlit.runtime.caching.cached_message_replay import (
     CachedMessageReplayContext,
     CachedResult,
     MsgData,
-    show_widget_replay_deprecation,
 )
 from streamlit.runtime.metrics_util import gather_metrics
 from streamlit.runtime.stats import CacheStat, CacheStatsProvider, group_stats
@@ -233,7 +233,6 @@ class CacheResourceAPI:
         show_spinner: bool | str = True,
         show_time: bool = False,
         validate: ValidateFunc | None = None,
-        experimental_allow_widgets: bool = False,
         hash_funcs: HashFuncsDict | None = None,
     ) -> Callable[[Callable[P, R]], CachedFunc[P, R]]: ...
 
@@ -246,7 +245,6 @@ class CacheResourceAPI:
         show_spinner: bool | str = True,
         show_time: bool = False,
         validate: ValidateFunc | None = None,
-        experimental_allow_widgets: bool = False,
         hash_funcs: HashFuncsDict | None = None,
     ) -> CachedFunc[P, R] | Callable[[Callable[P, R]], CachedFunc[P, R]]:
         return self._decorator(
@@ -256,7 +254,6 @@ class CacheResourceAPI:
             show_spinner=show_spinner,
             show_time=show_time,
             validate=validate,
-            experimental_allow_widgets=experimental_allow_widgets,
             hash_funcs=hash_funcs,
         )
 
@@ -269,7 +266,6 @@ class CacheResourceAPI:
         show_spinner: bool | str,
         show_time: bool = False,
         validate: ValidateFunc | None,
-        experimental_allow_widgets: bool,
         hash_funcs: HashFuncsDict | None = None,
     ) -> CachedFunc[P, R] | Callable[[Callable[P, R]], CachedFunc[P, R]]:
         """Decorator to cache functions that return global resources (e.g. database connections, ML models).
@@ -355,9 +351,6 @@ class CacheResourceAPI:
             is called to compute a new value. This is useful e.g. to check the
             health of database connections.
 
-        experimental_allow_widgets : bool
-            Allow widgets to be used in the cached function. Defaults to False.
-
         hash_funcs : dict or None
             Mapping of types or fully qualified names to hash functions.
             This is used to override the behavior of the hasher inside Streamlit's
@@ -366,12 +359,6 @@ class CacheResourceAPI:
             the provided function to generate a hash for it. See below for an example
             of how this can be used.
 
-        .. deprecated::
-            The cached widget replay functionality was removed in 1.38. Please
-            remove the ``experimental_allow_widgets`` parameter from your
-            caching decorators. This parameter will be removed in a future
-            version.
-
         Example
         -------
         >>> import streamlit as st
@@ -454,8 +441,6 @@ class CacheResourceAPI:
         ... def get_person_name(person: Person):
         ...     return person.name
         """
-        if experimental_allow_widgets:
-            show_widget_replay_deprecation("cache_resource")
 
         # Support passing the params via function decorator, e.g.
         # @st.cache_resource(show_spinner=False)
